/*Components: Arduino nano v3.0 supermini 328p old bootloader.
Pins:
SCL A5
SDA A4

Oled 128x64 address 0x3C

GPS TBS M8-2.
Tx to RX wiring

Button to reset Max speed on D7

Code from: https://www.youtube.com/watch?v=GKkvUyhLizI&ab_channel=HowToElectronics
*/

#include <SoftwareSerial.h>
#include <Wire.h>
#include <SSD1306Ascii.h>
#include <SSD1306AsciiWire.h>
#include <TinyGPS++.h>

// Define GPS serial connection (adjust pins if needed)
static const int RXPin = 4;  // GPS TX pin
static const int TXPin = 3;  // GPS RX pin (not really used here)
static const uint32_t GPSBaud = 9600;

// Create objects
SoftwareSerial mygps(RXPin, TXPin);
TinyGPSPlus gps;
SSD1306AsciiWire oled;

// Variables
float maxSpeed = 0;

// Timing
unsigned long lastUpdate = 0;
const unsigned long updateInterval = 500;  // update screen every 500ms

// Button setup
const int buttonPin = 7;  // Connect button between pin 7 and GND

void setup() {
  // Start Serial Monitor (optional, for debugging)
  Serial.begin(9600);

  // Start GPS serial
  mygps.begin(GPSBaud);

  // Start OLED
  Wire.begin();
  oled.begin(&Adafruit128x64, 0x3C);  // Address 0x3C is common
  oled.setFont(System5x7);
  oled.clear();
  oled.println("GPS");
  oled.println("Initializing");
  oled.println("...");

  // Button pin setup
  pinMode(buttonPin, INPUT_PULLUP);  // Use internal pull-up resistor
}

void loop() {
  boolean newData = false;
  // Read GPS data
  for (unsigned long start = millis(); millis() - start < 1000;) {
    while (mygps.available()) {
      if (gps.encode(mygps.read())) {
        newData = true;
      }
    }
  }


  //If newData is true
  if (newData == true) {
    newData = false;

    if (gps.location.isValid() == 1) {

      // Check button press
      if (digitalRead(buttonPin) == LOW) {  // Button pressed
        maxSpeed = 0;
      }

      // Update display every updateInterval milliseconds
      if (millis() - lastUpdate >= updateInterval) {
        lastUpdate = millis();

        //oled.clear(); // Clear display for fresh text

        oled.setCursor(0, 0);
        oled.print("Sats: ");
        oled.print(gps.satellites.value());
        oled.println("  ");

        oled.setCursor(0, 2 * 8);  // (line 2) 8 pixels per line
        oled.print("Speed: ");
        oled.print(gps.speed.kmph(), 1);  // 1 decimal place
        oled.println("   km/h  ");

        if (gps.speed.isValid()) {
          float currentSpeed = gps.speed.kmph();
          if (currentSpeed > maxSpeed) {
            maxSpeed = currentSpeed;
          }
        }

        oled.setCursor(0, 4 * 8);  // (line 4)
        oled.print("Max: ");
        oled.print(maxSpeed, 1);
        oled.println("   km/h  ");
      }
    }
  }
}